@@ -1,4 +1,5 @@
 import 'bot.dart';
+import 'commit_reference.dart';
 import 'util.dart';
 
 class Tag {
@@ -14,7 +15,7 @@ class Tag {
     requireArgumentNotNullOrEmpty(tagger, 'tagger');
   }
 
-  static Tag parseCatFile(String content) {
+  static Tag parseCatFile(String content, CommitReference ref) {
     final headers = <String, List<String>>{};
 
     final slr = StringLineReader(content);
@@ -32,11 +33,29 @@ class Tag {
       lastLine = slr.readNextLine()!;
     }
 
-    final objectSha = headers['object']!.single;
-    final type = headers['type']!.single;
-    final tag = headers['tag']!.single;
-    final tagger = headers['tagger']!.single;
+    String objectSha;
+    String type;
+    String tag;
+    String tagger;
+
+    if (headers['object'] != null) {
+      // Annotated Tag
+      objectSha = headers['object']!.single;
+      type = headers['type']!.single;
+      tag = headers['tag']!.single;
+      tagger = headers['tagger']!.single;
+    } else {
+      // Lightweight Tag
+      // https://git-scm.com/book/en/v2/Git-Basics-Tagging
+      objectSha = ref.sha;
+      type = 'lightweight';
+      final match = _refsTagsRegexp.firstMatch(ref.reference);
+      tag = match!.group(1)!;
+      tagger = headers['author']!.single;
+    }
 
     return Tag._internal(objectSha, type, tag, tagger);
   }
 }
+
+final _refsTagsRegexp = RegExp('refs/tags/(.*)');
